from __future__ import absolute_import
from __future__ import print_function
import sys
py_rumd_lib = {2:"../../../lib", 3:"../../../lib_py3"}[sys.version_info[0]]
sys.path.insert(1, py_rumd_lib)
sys.path.insert(0,"../../../Python")

import rumd
# it is essential that there is no doubt about which version is actually run!
print("File used to import the rumd module",rumd.__file__)

from rumd.Simulation import Simulation
from rumd.RUMD_Test import RUMD_Test

import string
import numpy
import os


class Test_Analysis (RUMD_Test):
    """
    Single-component LJ system with NVE dynamics.
    """
    def __init__(self):
        RUMD_Test.__init__(self)
        self.numParticlesList = [512]

        self.outfileTolList = {"energies_mean":0.02,
                               "energies_var":0.02,
                               "rdf":0.1,
                               "msd":0.05,
                               "Fs":0.02}

        
    def DifferencesSingleRun(self, mode):
        """ Overridden version of function which specifies how to compare runs
        """

        if mode != "short":
            raise ValueError("test_Analysis.DifferencesSingleRun: Wrong mode passed (somehow)")

        # make system calls to analysis functions since we compare their
        # output.
        
        self.CallAnalysisFunctions(self.ref_directory)

        differences = 0
        for filename in self.outfileTolList.keys():
            ref_file = open(filename+".ref")
            out_file = open(filename+".dat")
            ref_line = ref_file.readline()
            out_line = out_file.readline()
            lineNum = 0
            while ref_line:
                if not out_line:
                    self.outputfile.write("Missing lines in %s.dat\n" % filename)
                    differences += 1
                    break
                if ref_line.startswith("#"):
                    # test for equality
                    if out_line != ref_line:
                        self.outputfile.write("Comment line different in %s.dat\n" % filename)
                        differences += 1
                else:
                    ref_items = numpy.array([float(item) for
                                             item in ref_line.split()])
                    out_items = numpy.array([float(item) for
                                             item in out_line.split()])

                    if len(out_items) != len(ref_items):
                        self.outputfile.write("Inconsistent numbers of items in file %s.dat, line %d\n" % (filename, lineNum))
                        differences += 1
                        continue
                    #print filename, lineNum, max(abs(ref_items - out_items)) 
                    if max(abs(ref_items - out_items)) > self.outfileTolList[filename]:
                        self.outputfile.write("Numerical differences in file %s.dat, line %d\n" % (filename, lineNum) )
                        differences += 1
                    
                lineNum += 1
                ref_line = ref_file.readline()
                out_line = out_file.readline()
        print("differences", differences)
        return differences

    def CallAnalysisFunctions(self, directory):

        print(directory)
        print("../../../Tools/rumd_stats -d %s" % directory) 
        os.system("../../../Tools/rumd_stats -d %s" % directory )
        os.system("../../../Tools/rumd_rdf -d %s" % directory )
        os.system("../../../Tools/rumd_msd -d %s" % directory )


    def RunSimulation(self, mode, numParticles, pb, tp):


        if mode != "short":
            raise ValueError("Only \"short\" mode available for Analysis test")


        self.numTimeSteps = 1000 # needed for timing, although not meaningful
        
        if not self.generate_reference_data:
            # we don't run the simulation again since it's the analysis
            # programs that we're testing
            return


        
        # sample and simulation
        startConfg = "startN%d.xyz.gz" % numParticles

        
        sim = Simulation(startConfg, pb, tp, verbose=False)

        #if self.generate_reference_data:
        sim.SetOutputDirectory(self.ref_directory)
        #else:
        #    sim.SetOutputDirectory(self.out_directory)


        # integrator
        itg = rumd.IntegratorNVE(timeStep=0.005)

        # Create LJ pair-potential
        potential = rumd.Pot_LJ_12_6(cutoff_method=rumd.ShiftedForce)
        # Set up parameters for Lennard-Jones interactions ( Kob & Andersen )
        # i  j   Sig   Eps  Rcut (in units of Sigma_ij)
        potential.SetParams(0, 0, 1.0, 1.0, 1.5)
        potential.SetNbListSkin(0.4)
        
        sim.SetIntegrator(itg)
        sim.AddPotential(potential)

        sim.SetOutputScheduling("energies","linear",interval=10)

        # for testing tolerance, do some extra steps 
        sim.Run(5, suppressAllOutput=True)

        sim.Run(self.numTimeSteps)


        #if self.generate_reference_data:
        sim.sample.TerminateOutputManagers()
        self.CallAnalysisFunctions(self.ref_directory)
        # rename the files
        for filename in self.outfileTolList.keys():
            os.rename(filename+".dat",filename+".ref")
            

if __name__ == "__main__":

    rumd_test = Test_Analysis()

    # standard short test to be called from makefile
    # For generating reference data and running stress and performance modes
    # there are corresponding scripts to be submitted to pbs

    sys.exit(rumd_test.RunShort("gen_ref" in sys.argv ) )
    
    # for individual runs, could do for example
    #sys.exit(rumd_test.Test("short", 1000, 16, 8))
