#!/usr/bin/python3
#
#PBS -N test_280_tp_bug
#PBS -e test.error
#PBS -o test.log
#PBS -m ae
# this ensures PBS jobs run in the correct directory
import os, sys
if "PBS_O_WORKDIR" in os.environ:
    os.chdir(os.environ["PBS_O_WORKDIR"])
    sys.path.append(".")

# About this test:
# This can be used to test for the so-called 280 bug, which affects primarily
# large systems, when run on 280 cards with tp>1. We have not been able to
# identify the cause of the bug. It is manifested in three way (we cannot
# exclude that there is more than one bug, but this is probably not the case):
# (1) The simulation stops with an error generated by the card
# (2) The simulation stops producing output but does not terminate
# (3) Spikes appear in the energy and/or virial, for a single time step
# This script checks for such spikes in a simulation which runs with dt=0
# so that even small spikes (0.001 or more) and easily detected.

sys.path.insert(0,"../../../Swig")
sys.path.insert(0,"../../../Python")
sys.path.insert(0,"../../../Tools")

import rumdSimulation, rumd, rumd_tools
# it is essential that there is no doubt about which version is actually run!
print "File used to import the rumd module",rumd.__file__
print "File used to import the rumdSimulation module",rumdSimulation.__file__
print "File used to import the rumd_tools module",rumd_tools.__file__

from rumdSimulation import rumdSimulation
from RUMD_Test import RUMD_Test
from rumd import *

class CheckEnergySpikes:
    def __init__(self, init_pe, init_vir):
        self.last_pe = init_pe
        self.last_vir = init_vir
        self.counter = 0
    def ComparePE(self, sample):
        pe = sample.GetPotentialEnergy()/sample.GetNumberOfParticles()
        vir = sample.GetVirial()/sample.GetNumberOfParticles()
        pe_diff = pe - self.last_pe
        vir_diff = vir - self.last_vir
        if abs(pe_diff) > 1.e-3 or abs(vir_diff) > 1.e-3:
            raise ValueError, "spikes detected in potential energy/virial; step %d ,last_pe %f pe %f last vir %f vir %f" % (self.counter, self.last_pe, pe, self.last_vir, vir)
        self.last_pe = pe
        self.last_vir = vir
        self.counter += 1
        return "%.6f %.6f" % (pe_diff, vir_diff)

if True:
    sim = rumdSimulation("start.xyz.gz", pb=32, tp=4, allow_tp_280_large=True)

    itg = IntegratorNVT(timeStep=1.e-9, targetTemperature=1.2)
    sim.SetIntegrator(itg)


    pot = Pot_LJ_12_6(cutoff_method=ShiftedPotential)
    pot.SetNbListSkin(0.1)
    pot.SetParams(i=0, j=0, Epsilon=1.0, Sigma=1.0,  Rcut=14.0)
    sim.SetPotential(pot)
    sim.sample.CalcF()
    
    sim.sample.SetSortingScheme(rumd.SORT_XY)
    sim.SetSortInterval(1000)
    
    blockSize = 8192
    sim.SetBlockSize(blockSize)
    

    sim.SetOutputParameters("trajectory","none")
    sim.SetOutputParameters("energies","linear",interval=1)

    
    N = sim.sample.GetNumberOfParticles()

    check_obj = CheckEnergySpikes(sim.sample.GetPotentialEnergy()/N, sim.sample.GetVirial()/N)
    sim.NewOutputManager("check")
    sim.RegisterCallback("check", check_obj.ComparePE, header="pe_diff vir_diff")
    sim.SetOutputParameters("check","linear",interval=1)

    sim.Run(4*blockSize)
